#ifndef MAT4F_H
#define MAT4F_H

#define MAT4F_DIM 4
#define MAT4F_NUM_ELEM 16

#include <memory>
#include <initializer_list>
#include <array>
#include <stdexcept>	// std::out_of_range exception

#include <algorithm>
#include <iterator>
#include <iostream>

// Stores a 4 by 4 Matrix in Row Major order.
//	[ 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14 ] ->

//	1  2  3  4
//	5  6  7  8
//	9  10 11 12
//	13 14 15 16

class Mat4f
{

typedef std::array< float, MAT4F_NUM_ELEM > ARRAY_16f;
typedef std::unique_ptr< ARRAY_16f > Mat4fHandle;

public:
	explicit Mat4f();
	
	// not explicit, so Mat4f m = {1,...,16};
	Mat4f( std::initializer_list< float > list );
	// Move constructor
	Mat4f( Mat4f&& moved );
	// Copy Constructor
	Mat4f( const Mat4f & copied );
	~Mat4f();

	Mat4f & operator=( const Mat4f & copied );
	Mat4f & operator=( Mat4f && moved );

	// To access individual row, coloum entry
	float & operator()( int row, int column );
	float operator()( int row, int column ) const;

	// To access individual element in linear array
	float & operator[]( int element );	
	float operator[]( int element ) const;

	// TODO
	/*
	Mat4f operator+( Mat4f other ) const;
	Mat4f operator*( const Mat4f & other ) const;
	Mat4f operator*( float scalar ) const;
	*/
	
	// Some helper functions
	bool isValidDimIndex( int idx ) const;
	bool isValidElementIndex( int idx ) const;

	ARRAY_16f::iterator begin();
	ARRAY_16f::iterator end();
	ARRAY_16f::const_iterator begin() const;
	ARRAY_16f::const_iterator end() const;

	float const * data() const;
		
private:
	Mat4fHandle m_ptr;

};

// So you can std::cout << mat << std::endl; print the content
// change this if you want to print out row by row
std::ostream & operator<<( std::ostream &, const Mat4f & mat );


#endif // MAT4F_H
